import Utils from './../API/IKUtils';
//
export default class IKDBBaseExpression  {
  constructor() {
    this.values = [];
    this.operator = IKDBExpressionOperator.AND;
    this.counter = 0;
  }
  /*public methods*/
  appendOperator(key, val, cond) {
    const aKey = this._anonimizeKey(key);
    this.values.push({exp: `${key} ${cond} ${aKey}`, values: [val], appendKeys: [aKey]});
  }
  appendBetween(key, val1, val2) {
    const aKey = this._anonimizeKey(key);
    const aKey2 = this._anonimizeKey(key);
    this.values.push({exp: `(${key} BETWEEN ${aKey} AND ${aKey2})`, values: [val1, val2], appendKeys: [aKey, aKey2] });
  }
  appendBeginsWith(key, val) {
    const aKey = this._anonimizeKey(key);
    this.values.push({exp: `begins_with(${key}, ${aKey})`, values: [val], appendKeys: [aKey]});
  }
  appendAttributeType(key, val) {
    const aKey = this._anonimizeKey(key);
    this.values.push({exp: `attribute_type(${key}, ${aKey})`, values: [val], appendKeys: [aKey]});
  }
  appendContains(key, val) {
    const aKey = this._anonimizeKey(key);
    this.values.push({exp: `contains(${key}, ${aKey})`, values: [val], appendKeys: [aKey]});
  }
  appendExists(key) { this.values.push({exp: `attribute_exists(${key})` }); }
  appendNotExists(key) { this.values.push({exp: `attribute_not_exists(${key})` }); }
  appendIN(key, values) {
    let aKeys = values.map((val)=>this._anonimizeKey(val));
    this.values.push({exp: `(${key} IN (${aKeys.toString()}))`, values: values, appendKeys: aKeys });
  }
  _anonimizeKey(_key) {
    this.counter++;
    return (':' + _key + this.counter);
  }
  //Groups
  startGroupping() { this.values.push({startGroupping: true}); }
  endGroupping() { this.values.push({endGroupping: true}); }
  addSeparator(val) { this.values.push({exp: ` ${val} `}); }
  //Builds items and exp (if set)
  buildExpression() {
    const exp = this.__buildExpressionOnQuery();
    console.debug('Building query expression..', exp);
    return exp;
  }
  __buildExpressionOnQuery() {
    let expression = {};
    if (this.values.length > 0) {
      //Check required values
      if (!expression.items) expression.items = {};
      //build conditional expression
      let condition = '';
      let groupping = false;
      let keyCount = 0;
      for (let addObj of this.values) {
        if (condition != '' && !groupping) condition += ` ${this.operator} `;
        //Check for include types
        if (addObj.startGroupping) {
          condition += `(`;
          groupping = true;
        } else if (addObj.endGroupping) {
          condition += `)`;
          groupping = false;
        } else if (addObj.values) { //multi value expressions
          condition += `${addObj.exp}`;
          let i = 0;
          for (let val of addObj.values) {
            expression.items[`${addObj.appendKeys[i]}`] = Utils.encapsulateForDB(val);
            i++;
          }
        } else if (addObj.exp) { //pure expressions
          condition += `${addObj.exp}`;
        }
      } expression.exp = condition;
    } return expression;
  }
}
//
export const IKDBExpressionOperator = {
  EQUAL: '=',
  NOT_EQUAL: '<>',
  LESS_THAN: '<',
  LESS_EQUAL_THAN: '<=',
  GREATER_THAN: '>',
  GREATE_EQUAL_THAN: '>=',
  NOT: 'NOT',
  AND: 'AND',
  OR: 'OR'
};
